<?php

if (!defined('ABSPATH'))
	die();
if (!class_exists('WishListMemberUserSearch')) {
	require_once(ABSPATH . '/wp-admin/includes/user.php');
	if (get_bloginfo('version') >= 3) { //start check for wp3
		if (!class_exists('WP_User_Search')) {
			require_once(ABSPATH . '/wp-admin/includes/deprecated.php');
		}

		class WishListMemberUserSearch extends WP_User_Search {

			function WishListMemberUserSearch($search_term = '', $page = '', $role = '', $ids = '', $sortby = '', $sortorder = '', $howmany = 15, $more_filters = array()) {
				if (is_array($ids)) {
					$this->IDs = (array) $ids;
					$this->IDs[] = 0;
					$this->IDs = array_unique($this->IDs);
				} else {
					$this->IDs = '';
				}
				$this->SortBy = $sortby;
				$this->SortOrder = $sortorder ? $sortorder : 'ASC';
				$this->users_per_page = $howmany;
				$this->additional_filters = $more_filters;
				$this->WP_User_Search($search_term, $page, $role);
			}

			function prepare_query() {
				global $wpdb;

				$this->first_user = ($this->page - 1) * $this->users_per_page;

				$this->query_limit = $wpdb->prepare(" LIMIT %d, %d", $this->first_user, $this->users_per_page);

				$search_sql = '';
				if ($this->search_term) {
					$searches = array();
					$search_sql = 'AND (';
					foreach (array('user_login', 'user_nicename', 'user_email', 'user_url', 'display_name') as $col)
						$searches[] = $col . " LIKE '%$this->search_term%'";
					$search_sql .= implode(' OR ', $searches);
					$search_sql .= ')';
				}

				$this->query_from = " FROM $wpdb->users";
				$this->query_where = " WHERE 1=1 $search_sql";

				if ($this->IDs) {
					$x = $this->IDs;
					if ((string) $x[0] == '-') {
						unset($x[0]);
						$in = 'NOT IN';
					} else {
						$in = 'IN';
					}
					$key = array_search('', $x, true);
					if ($key !== false)
						unset($x[$key]);
					$x = implode(',', $x);
					$this->query_where.=" AND `ID` {$in} ({$x}) ";
				}

				if ($this->SortBy) {
					$this->query_orderby = " ORDER BY {$this->SortBy} {$this->SortOrder}";
				}

				if (!$this->users_per_page)
					$this->query_limit = '';
			}

			function query() {
				global $wpdb;
				global $WishListMemberInstance;

				// We will only sort by level registration date if we're filtering by membership level
				$level = $this->additional_filters['level'];

				if((!empty($level)) && ($level != 'incomplete') && ($level != 'nonmembers')) {

					if ($this->search_term) {
						$searches = array();
						$term_search = '(';
						foreach (array('user_login', 'user_nicename', 'user_email', 'user_url', 'display_name') as $col)
							$searches[] = $col . " LIKE '%$this->search_term%'";
						$term_search .= implode(' OR ', $searches);
						$term_search .= ')';

						$search_sql[] = $term_search;
					}
					$search_sql[] = $wpdb->prepare("ul.level_id=%d", $level);
					if(!empty($search_sql)) {
						$search_sql = ' WHERE ' . implode(' AND ' , $search_sql);
					}

					$this->query_from = " FROM $wpdb->users"
					." LEFT JOIN {$WishListMemberInstance->Tables->userlevels} ul on ($wpdb->users.ID=ul.user_id)"
					." LEFT JOIN {$WishListMemberInstance->Tables->userlevel_options} ulo on (ulo.userlevel_id=ul.ID)"
					." LEFT JOIN {$WishListMemberInstance->Tables->user_options} uo on ($wpdb->users.ID=uo.user_id)";
					$this->query_where = "$search_sql";

					$unprocessed_data = $wpdb->get_results("SELECT DISTINCT($wpdb->users.ID)" . $this->query_from . $this->query_where . $this->query_orderby . $this->query_limit, ARRAY_A);

					$levels_data = array();

					// loop through results and convert date to timestamp for easier sorting
					foreach($unprocessed_data as $data) {
						$levels_data[$data['ID']] = $data['ID'];
					}
					
					if($this->SortOrder == 'ASC')
						ksort($levels_data);
					else
						krsort($levels_data);

					$this->results = $levels_data;
				} else {
					$this->results = $wpdb->get_col("SELECT DISTINCT($wpdb->users.ID)" . $this->query_from . $this->query_where . $this->query_orderby . $this->query_limit);
				}

				if ( $this->results )
					$this->total_users_for_query = $wpdb->get_var("SELECT COUNT(DISTINCT($wpdb->users.ID))" . $this->query_from . $this->query_where); // no limit
				else
					$this->search_errors = new WP_Error('no_matching_users_found', __('No matching users were found!'));
			}

		}

		// end check for wp3
	}else {

		// start wp2.9- this section is for wp2.9 -

		class WishListMemberUserSearch extends WP_User_Search {

			function WishListMemberUserSearch($search_term = '', $page = '', $role = '', $ids = '', $sortby = '', $sortorder = '', $howmany = 15) {
				if (is_array($ids)) {
					$this->IDs = (array) $ids;
					$this->IDs[] = 0;
					$this->IDs = array_unique($this->IDs);
				} else {
					$this->IDs = '';
				}
				$this->SortBy = $sortby;
				$this->SortOrder = $sortorder ? $sortorder : 'ASC';

				$this->users_per_page = $howmany;

				$this->WP_User_Search($search_term, $page, $role);
			}

			function prepare_query() {
				parent::prepare_query();
				if ($this->IDs) {
					$x = $this->IDs;
					if ((string) $x[0] == '-') {
						unset($x[0]);
						$in = 'NOT IN';
					} else {
						$in = 'IN';
					}
					$key = array_search('', $x, true);
					if ($key !== false)
						unset($x[$key]);
					$x = implode(',', $x);
					$this->query_from_where.=" AND `ID` {$in} ({$x}) ";
				}
				if ($this->SortBy) {
					$this->query_sort = " ORDER BY {$this->SortBy} {$this->SortOrder}";
				}
				if (!$this->users_per_page)
					$this->query_limit = '';
			}

		}

		// end wp2.9
	}
}
?>